%  program to do simulations for "response to Lunqvist and Uhlig"
% differential equation solutions are in LU_differential_equations_main

close all
clear all

% ---------------------
% PART II -- SIMULATIONS
% ----------------------

% redo simulations or play with subsidiary calculations and plots only?

redosim = 0; % 0, 1, or 2. 
        % 0: loads last run of 1 and plays with graphs. you must have run 1
        % before so the results can be loaded. It will overwrite graphs
        % 1: redo production simulation, save workspace. Also rewrites graphs
        % 2: redoes simulation, you can play with parameters. Will not save
        % workspace or overwrite graphs. 

if redosim == 0;
    load('LU_simulation_workspace');
elseif redosim == 1;
    % choices DON"T MESS WITH THESE. These choices are the production
    % values. 
    N = 1000; % number of simulations to take E
    disp('Redoing simulation. This will take a while')
    h = 12; % horizon; 12 = months.
    nsteps = 30; % how much to subdivide the month.
    Ty = 50; % how far out to take sums in years. Is multiplied by h and nsteps below
    psiv = [-0.1:0.01:0.1 ]'/100; % vector of endowment destructions in month one. - = destruction, + = manna
    Np = size(psiv,1); % number of psis being tried
    ind0 = find(psiv == 0); % index of zero for later
    
    % Warning: large positive psi will lead to s= smax, and then a deep fall from which it takes a long
    % time to recover. It gives the appearance that extreme positive psi lower welfare.
    % But I think this is  due to numerical problems very near s max. If
    % you want those, improve numerical stability. Make plots of simulated time seres of s to make sure they make sense
   
    
    % parameters
    g = 0.0189;
    sc = 0.0150;
    phi = 0.87;
    gamma = 2.00;
    delta = 0.89;
    Sbar = 0.057;
    sbar = log(Sbar);
    smax = sbar + 1/2*(1-Sbar^2);
    Smax = exp(smax);
    
    % all symbols in the paper are at annual horizon.
    % scaling rule for horizons h
    %g = g/h;
    %sc = sc/h^0.5;
    %phi = phi^(1/h);
    %delta = delta^(1/h);
    
    [ EW_CC, se_EW_CC,se_EW_CC_diff,EW_LU, se_EW_LU,se_EW_LU_diff,st_CC, st_LU,W_CC, W_LU,ct_LU,ct_CC]...
      = dosim( psiv,Ty,h,N,g,sc,phi,gamma,delta,Sbar,sbar,smax,Smax);
    
     save('LU_simulation_workspace');
elseif redosim == 2; % dosimulation, do not save workspace, do not save plots
    % choices
    N = 100; % number of simulations to take E
    h = 12; % horizon; 12 = months.
    nsteps = 30; % how much to subdivide the month.
    Ty = 50; % how far out to take sums in years. Is multiplied by h and nsteps below
    psiv = [-20:1:20 ]'/100; % vector of endowment destructions in month one. - = destruction, + = manna
    Np = size(psiv,1); % number of psis being tried
    ind0 = find(psiv == 0); % index of zero for later    
    
    % parameters
    g = 0.0189;
    sc = 0.0150;
    phi = 0.87;
    gamma = 2.00;
    delta = 0.89;
    Sbar = 0.057;
    sbar = log(Sbar);
    smax = sbar + 1/2*(1-Sbar^2);
    Smax = exp(smax);
    
    [ EW_CC, se_EW_CC,se_EW_CC_diff,EW_LU, se_EW_LU,se_EW_LU_diff,st_CC, st_LU,W_CC, W_LU,ct_LU,ct_CC]...
      = dosim( psiv,Ty,h,N,g,sc,phi,gamma,delta,Sbar,sbar,smax,Smax);
    
     save('LU_simulation_workspace_trash'); % its saved but no code uses it now
    
end;% ends if redosim

% now plots 

figure; 
hold on;
y1 = EW_LU-EW_LU(ind0);
plot(100*psiv,y1, 'linewidth', 2);
y2 = EW_CC-EW_CC(ind0);
plot(100*psiv,y2,'--r','linewidth', 2);
if 0; % standard error bars only for internal use, if 0 for production  
    % absolute errors are distressingly large even for 10,000 simulations. But the
    % graph always looks the same, even with 10 simulations. These differenced errors are quite tight
    % So I think
    % we're meausring the pattern across psi and the difference with great
    % accuracy. 
    plot(100*psiv,y1+se_EW_LU_diff,'--b')
    plot(100*psiv,y1-se_EW_LU_diff,'--b')
    plot(100*psiv,y2+se_EW_CC_diff,'--r')
    plot(100*psiv,y2-se_EW_CC_diff,'--r')
end; 
plot(100*[min(psiv) max(psiv)]',[0 0]','-k'); % axis lines
plot([0 0]',[min(min(y1),min(y2)) max(max(y1),max(y2))]','--k'); 
xlabel('\psi, month 1 log consumption, percent'); 
ylabel('Utility - U(\psi=0)')
legend('One month \Delta','One day \Delta',4)
if (redosim == 0) | (redosim == 1);
    axis([-0.1 0.1 -0.01 0.025]);
    print -depsc2 Uhlig_1.eps;
end;


% plots a few draws reassurance that the shape across phi and comparison is
% stable even if the draws are not that stable. 
figure; 
hold on;
y1 = W_LU(:,1:min(10,N))-ones(Np,1)*W_LU(ind0,1:min(10,N));
plot(100*psiv,y1, 'linewidth', 2);
y2 = W_CC(:,1:min(10,N))-ones(Np,1)*W_CC(ind0,1:min(10,N));
plot(100*psiv,y2,'--r','linewidth', 2);
plot(100*[min(psiv) max(psiv)]',[0 0]','-k'); % axis lines


% plot initial time series to see what's going on
% S = (C-X)/C = 1-X/C
% X=C(1-S)

xt_LU = ct_LU + log(1-exp(st_LU));
xt_CC = ct_CC + log(1-exp(st_CC));
marker = 8; % marker size for plot

figure
subplot(2,2,1);
hold on
y=100*(ct_LU([1 ind0 end],1:3)'-g/h*(-1:1)'*ones(1,3));
plot((0:2),y(:,1),'-rv','linewidth',2,'MarkerSize',marker);
plot((0:2),y(:,2),'-b','linewidth',2);
plot((0:2),y(:,3),'--go','linewidth',2,'MarkerSize',marker);
title('c_t - gt, monthly');
xlabel('t, months'); 
ylabel('percent');

subplot(2,2,2);
hold on
y=100*(xt_LU([1 ind0 end],1:3)'-g/h*(-1:1)'*ones(1,3));
plot((0:2),y(:,1),'-rv','linewidth',2,'MarkerSize',marker);
plot((0:2),y(:,2),'-b','linewidth',2);
plot((0:2),y(:,3),'--go','linewidth',2,'MarkerSize',marker);
title('x_t - gt, monthly');
xlabel('t, months'); 
ylabel('percent');

subplot(2,2,3);
hold on
y=100*(ct_CC([1 ind0 end],1:61)'-g/h*(-1:1/30:1)'*ones(1,3));
plot((0:1/30:2),y(:,1),'-r','linewidth',2);
plot((0:1/30:2),y(:,2),'-b','linewidth',2);
plot((0:1/30:2),y(:,3),'--g','linewidth',2);
plot([0 1 2]',y([1 31 61],1),'rv','MarkerSize',marker);
plot([0 1 2]',y([1 31 61],3),'go','MarkerSize',marker);
title('c_t - gt, daily');
xlabel('t, months'); 
ylabel('percent');

subplot(2,2,4);
hold on
y=100*(xt_CC([1 ind0 end],1:61)'-g/h*(-1:1/30:1)'*ones(1,3));
plot((0:1/30:2),y(:,1),'-r','linewidth',2);
plot((0:1/30:2),y(:,2),'-b','linewidth',2);
plot((0:1/30:2),y(:,3),'--g','linewidth',2);
plot([0 1 2]',y([1 31 61],1),'rv','MarkerSize',marker);
plot([0 1 2]',y([1 31 61],3),'go','MarkerSize',marker);
title('x_t - gt, daily');
xlabel('t, months'); 
ylabel('percent');

if (redosim == 0) | (redosim == 1);
    print -depsc2 uhlig_2.eps
end

