function [ EW_CC, se_EW_CC,se_EW_CC_diff,EW_LU, se_EW_LU,se_EW_LU_diff,st_CC, st_LU,W_CC, W_LU,ct_LU,ct_CC]...
      = dosim( psiv,Ty,h,N,g,sc,phi,gamma,delta,Sbar,sbar,smax,Smax);
% function to do simulations called by uhlig.m

    Np = size(psiv,1); % number of psis being tried
    ind0 = find(psiv == 0); % index of zero for later
    W_LU = zeros(Np,N); % space for utility in each trial
    W_CC = zeros(Np,N); % space for utility in each trial
    
    for trial = 1:N;
        
        % both simulations use the same shocks so that simualtion error is common
        % to both, makes comparison more accurate even if absolute numbers are less.
        % shocks for high frequency simulation, then average to monthy
     
        % all values of psi use the same draw of v, so errors are common as
        % you vary psi. Again, this means the comparison across psi is much
        % smoother than the sampling error -- be careful! Smooth across psi
        % does not mean well measured! 
        
        nstepsd = 30; % how many steps to subdivide for CC simulation.
        
        Td = Ty * nstepsd * h + 1; % time 0; day 1-30; day 31-60; day 61 on 
        Tm = Ty * h + 1;% time 0, 1, 2, then stochastic at 3 and beyond
        vd = sc/(nstepsd*h)^0.5*randn(1,Td-2*nstepsd-1); % daily shocks from index 62 on
        % make monthly shocks the sum of daily shocks.
        for indx=1:Tm-3;
            vm(indx)=sum(vd(nstepsd*(indx-1)+1:nstepsd*indx),2);
        end;
        
        st_LU = zeros(Np,Tm);    % place to store time series of s
        ct_LU = zeros(Np,Tm);    % space for consumption
        st_CC = zeros(Np,Td);    % place to store time series of s
        ct_CC = zeros(Np,Td);    % space for consumption
        
        % the first two months are nonstochastic, first adding psi then getting
        % rid of it.
        
        % LU: one month of endowment destruction
        ct_LU(:,1) = -ones(Np,1)*g/h ; % time 0
        ct_LU(:,2) = psiv; % time 1
        ct_LU(:,3) = ones(Np,1)*g/h; % time 2
        
        % CC: V shaped
        % want to end up at t = nsteps, consumption = psi, 
        % baseline is -g/h at t=0; 0 at t = 1/h
        ct_CC(:,1) = -ones(Np,1)*g/h; %day 30 of month 0
        ct_CC(:,2:nstepsd+1) = ones(Np,1)*(-1+(1:nstepsd)/(nstepsd))*g/h + psiv*(1:nstepsd)/(nstepsd); % day 1 through 30
        % next get back from psi to g/h
        ct_CC(:,nstepsd+2:2*nstepsd+1) = ones(Np,1)*((1:nstepsd)/(nstepsd))*g/h + psiv*(1-(1:nstepsd)/(nstepsd));
        
        % after that c is just the usual normal
        ct_LU(:,3:Tm) = cumsum([ct_LU(:,3) (ones(Np,1)*vm + g/h)],2);
        ct_CC(:,2*nstepsd+1:Td) = cumsum([ct_CC(:,2*nstepsd+1) (ones(Np,1)*vd + g/(h*nstepsd))],2);
        
        % now find s(t); start at sbar
        % this is inefficient, but clear. you could do pre 2*nsteps and post in
        % separate steps and use v not c if you need speed
        % also discretizing teh whole lifetime at daily interval is clearly
        % inefficient.
        
        % c(0) is -g/h. hence last term in next equation
        st_CC(:,1) = sbar;
        for t = 2:Td;
            st_CC(:,t) = sbar + phi^(1/(nstepsd*h))*(st_CC(:,t-1)-sbar) + lambdaofs(st_CC(:,t-1),sbar,smax).*(ct_CC(:,t)-ct_CC(:,t-1)-g/(h*nstepsd));
        end;
        
        st_LU(:,1) = sbar;
        for t = 2:Tm;
            st_LU(:,t) = sbar + phi^(1/h)*(st_LU(:,t-1)-sbar) + lambdaofs(st_LU(:,t-1),sbar,smax).*(ct_LU(:,t)-ct_LU(:,t-1)-g/h);
        end;
        
        % find achieved utility adds dt so the magintues are about the same.
        
        W_CC(:,trial) = 1/(1-gamma)*sum( ones(Np,1)*delta.^((1:Td)/(h*nstepsd)).*exp((1-gamma)*(ct_CC+st_CC))*(1/(h*nstepsd)),2);
        W_LU(:,trial) = 1/(1-gamma)*sum( ones(Np,1)*delta.^((1:Tm)/h).*exp((1-gamma)*(ct_LU+st_LU))*1/h,2);
    end;
    
    EW_CC = mean(W_CC,2); % take expected value of utility
    se_EW_CC = std(W_CC,0,2)/N^0.5; % standard error to evaluate simulation accuracy
    se_EW_CC_diff = std(W_CC-ones(Np,1)*W_CC(ind0,:),0,2)/N^0.5; % standard error of differences to psi 0 value -- accuracy of graph
   
    
    EW_LU = mean(W_LU,2); % take expected value of utility
    se_EW_LU = std(W_LU,0,2)/N^0.5; % standard error to evaluate simulation accuracy
    se_EW_LU_diff = std(W_LU-ones(Np,1)*W_LU(ind0,:),0,2)/N^0.5; % standard error of differences to psi 0 value -- accuracy of graph 
   
    % if you plot(W_CC'); hold on; plot(W_LU') you will see that the W are
    % almost perfectly correalted. The level of W is not well measured, but
    % the differences betweeh LU and CC are very small. 
    
    % Plot also reveals that the W have a big skew -- a few outliers on
    % very low W. I don't know what deep lesson that holds. W may not have
    % a second moment? 
   
end

